/********************************************************************

    serialxmem.c

    Z-World, 2004

	 Sample program for new, abstracted RS232.LIB library.

    Demonstrates use of the new serXopen2() functions to
    use xmem buffers for the the serial port read and write
    buffers.

    Hardware setup:

    connect TXC-RXB
            TXB-RXC
    or change use different serialports and change the
    function calls.

********************************************************************/

// Uncomment only one
//#define BAUDR  19200L
#define BAUDR  57600L

#define BUFFERSIZE 0x1008
#define NBUFF 4

// Uncomment to use root buffers
// and serXopen, othewise xmem
// buffers will be used.
//#define USEROOTBUFFERS

#ifdef USEROOTBUFFERS
#define BINBUFSIZE   BUFFERSIZE
#define BOUTBUFSIZE  BUFFERSIZE
#define CINBUFSIZE   BUFFERSIZE
#define COUTBUFSIZE  BUFFERSIZE
#endif

// Set ISR optimizations
#define SERIAL_N81_MODE_ONLY
#define SERIAL_NO_FLOWCONTROL

// needed for some boards, but already defined for
// stock boards that need it if Sys ID block intact
//#define SERB_USEPORTD

main(){
	char c,buf[100];
	int n, i, j, sz;
	unsigned long paddr[40];

#ifdef USEROOTBUFFERS
   // Open ports with static root data buffers,
   // done at compile time (use of serXopen() and NOT
   // the USEROOTBUFFERS macro trigger the
   // buffer compile time allocations.

   if ((n=serBopen(BAUDR))!=1)
   {
		printf("Error %d opening serial port B\n",n);
   }
   if ((n=serCopen(BAUDR))!=1)
   {
		printf("Error %d opening serial port C\n",n);
   }
#else
    // Open ports with xmem buffers

    // This technique aligns the first buffer on a 0x100 boundary.
    // 12 buffers 0x1008 can safely be allocated like this
    // without any of them crossing 0xFFFF after they are
    // normalized to 0xEnnn range addresses. This is only necessary
    // for buffers of 0x1008 bytes, since buffers of 0x808 cannot
    // cross 0xFFFF if they start anywhere in the 0xEnnn range.
    paddr[0] = xalloc(BUFFERSIZE*NBUFF+0x100);
    paddr[0] = paddr[0]&0xFFFFFF00L;
	 for (i=1; i<NBUFF; i++)
    {
	    paddr[i] = paddr[i-1] + BUFFERSIZE;
    }

   // The macro XPTR_EOFFSET gives the 16 bit Exxx range,
   // normalized address of a long physical address.
   // XPTR_XPC gives its 8 bit XPC segment value
   // normalized address of a long physical address.

	if((n=serBopen2(BAUDR,XPTR_EOFFSET(paddr[0]),
   					 XPTR_XPC(paddr[0]),0x1008,
                   XPTR_EOFFSET(paddr[1]),
                   XPTR_XPC(paddr[1]),0x1008)))
	{
		printf("Error %d opening serial port B\n",n);
   }
	if((n=serCopen2(BAUDR,XPTR_EOFFSET(paddr[2]),
                   XPTR_XPC(paddr[2]),0x1008,
                   XPTR_EOFFSET(paddr[3]),
                   XPTR_XPC(paddr[3]),0x1008)))
   {
		printf("Error %d opening serial port C\n",n);
	}

#endif

// Uncomment these macros to simulate serial port
// hardware. The second one must stay after the
// open calls to work.
//#define BUFF_ONLY_NO_HW_TMODE
//SERIAL_SIMPORTS(C,B);

	n = 25;

#define MESSAGE "Buy more Rabbits!"

	while(1){
		if( (i=serBgetError()) || (j = serCgetError()))
	   {
   	 	printf("error %d, %d\n",i,j);
	   }
   	costate{
	   	wfd cof_serBputc((++n%26)+0x41);
   	   wfd c=cof_serCgetc();
    		wfd cof_serBputs(MESSAGE);
	    	wfd cof_serCgets(buf, strlen(MESSAGE), 400);
		  	printf("%c %s\n",c,buf);
	   }
	}
}













































