/**************************************************************

	 xmalloctest.c 	 Z-World,  2004

    This program exercises the xmalloc dynamic xmemory
    allocation system. It initializes a heap for a randomly
    chosen number of objects of like random size, allocates the
    full number of objects, then frees them all and makes sure
    the heap returns to initial conditions.

    This is a good test program if making modifications to
    the xheap system.

    Please read the comments at the top of XMALLOC.LIB
***************************************************************/

#use "RAND.LIB"
#use "XMALLOC.LIB"

#define INITRSTR(x)  for(i=0;i<x;i++)rstr[i]=i+1;rstr[i]=0

#define  MINOBJSIZE 1
#define  MAXOBJSIZE 0x200
#define  MINNUMOBJ  2
#define  MAXNUMOBJ  20


main(){
   long sz,paddr[MAXNUMOBJ];
	int i,j,OSIZE,NOBJ;
	char rstr[MAXNUMOBJ+1];
	char str1[64];
	char str2[64];

	rand16_init(15 & (int)MS_TIMER);  // initialize rand

	while(1)  // do forever or until error
   {
   	j++;

   	// choose random # of objects and object size
		OSIZE = ranRange(MINOBJSIZE,MAXOBJSIZE);
		NOBJ  = ranRange(MINNUMOBJ,MAXNUMOBJ);

		xheapInit(OSIZE,NOBJ);  // initialized xheap

		outputXFL(str1);        // save initial heap attributes

		INITRSTR(NOBJ);         // initialize string used for random
                              //   selection order.

		// allocate buffers of random size in random order
		for (i=0;i<NOBJ;i++)
		{
			// alternate between allocating random size <- OSIZE buffers
         // and OSIZE buffers (to better exercise xfree())
		   if(j&1)paddr[dealRandom(rstr)-1] =
         		xmalloc(ranRange(1,OSIZE));
		   else paddr[dealRandom(rstr)-1] =
         		xmalloc(OSIZE);
		}

		INITRSTR(NOBJ);    		//  reinitialize random string

		// free buffers in random order
		for (i=0;i<NOBJ;i++)
		{
		   xfree(paddr[dealRandom(rstr)-1]);
		}
		outputXFL(str2); 			// save current heap attributes

		if(!strcmp(str1,str2))  // since all buffers were freed,
		{                       //  the heap attributes should
	   	printf("%s",str1);   //  match the initial attributes
			printf("%s",str2);
			printf("\n");
   		xHeapFree();         // free the whole heap
		}
		else
		{
		  printf("ERROR!");
		  break;
		}
	}
}

dealRandom(char* str)
{
	// takes a NULL terminated string of arbitrary
   // characters, removes one character from a
   // random index location, shortens the string
   // by one, and returns the value removed.
   int len,indx;
   char *ptr, retval;

   len = strlen(str);
   indx = rand16() % len;
   retval = str[indx];
   str[indx] = 0;
   ptr = &str[indx+1];
   strcat(str,ptr);
   return retval;
}

ranRange(int min, int max)
{
	// return a random number between min & max
	return min + (rand16() % (1+max-min));
}

outputXFL(char * str)
{
	auto unsigned long paddr;
	auto struct xfree__list block;
   auto int i;

	paddr = GetFirstEntry();
	xmem2root(&block,paddr,XENTRYSIZE);

   // uncomment to print out entire free list
   /*
   for (i=0;i<_xNblocks;i++)
   {
		xmem2root(&block,paddr,XENTRYSIZE);
 			printf("L= %08lx, A= %08lx, S= %08lx, N= %08lx\n",paddr,
                block.start, block.Size, block.next);
		paddr = block.next;
   }
   printf("\n");
   */

   if(str)sprintf(str,"A= %08lx, S= %08lx, N= %08lx\n",
                  block.start, block.Size, block.next);
}


