/*****************************************************

	xstring2.c
	Z-World, Inc, 2004

	Example of using xstring.

	This program demonstrates how to access the 	strings
   of an xstring declaration with variable string sizes.

   It also demonstrates the new %ls printf/sprintf format
   specifier that allows direct printing of string at
   long addresses without copying them to root. 

   There are no limitations on table or string size other
   than available RAM and flash, and 64K bytes for the
   max length of single string or the total table.
   A function xstrTableInit is called once at
   the begining of the program to create lookup by index
   table in extend RAM that includes checksums for the
   entries.

	See the Dynamic C manual for more information
	about how Dynamic C stores xstrings in XMEM.
****************************************************/

xstring mystrings {

/*  1 */   "This program demonstrates how to access the strings",
/*  2 */   "of an xstring declaration with variable string sizes.",
/*  3 */   "There are no limitations on table or string size other",
/*  4 */   "than available RAM and flash, and 64K bytes for the",
/*  5 */   "max length of single string or the total table",
/*  6 */   "A function xstrTableInit is called once at",
/*  7 */   "the begining of the program to create lookup by index",
/*  8 */   "table in extend RAM.",
/*  8 */   " ",
/* 10 */   "See the Dynamic C manual for more information",
/* 11 */   "about how Dynamic C stores xstrings in XMEM"
         };

// Stucture for a look-up table entry entry
typedef struct
{
	unsigned long padd;  // physical address
   unsigned int len;    // length
	char     checksum;   // checksum for entry
} xtableEntry;


int xstrTableInit(unsigned long *xstrLUTableAddr,
							  unsigned long xtring_addr,
                       int flags);

int xstrLookup(char* data, unsigned long *xstrLUTabAddr,
               unsigned index, int maxsize);

long xstrGet(unsigned long *xstrLUTabAddr,
               unsigned index, int maxsize);

void main()
{
   unsigned long xstrLUTableAddr;
	char buffer[255];
   int i, retv;
   long paddr;

  	xstrTableInit(&xstrLUTableAddr, mystrings,0);

   for(i= 0; i<11; i++)
   {
		// copy to root and print
	   if( xstrLookup(buffer,&xstrLUTableAddr,i,sizeof(buffer)))
      {
			printf("Error #%d while looking up string\n",retv);
      }
      else
      {
			printf("%s\n",buffer);
      }

	   if( (paddr = xstrGet(&xstrLUTableAddr,i,sizeof(buffer))) < 0 )
      {
			printf("Error #%d while looking up string\n",(int)paddr);
      }
      else
      {
			// print with %ls without copying to root
			printf("%ls\n",paddr);

      }
	}
   timeit();
}


nodebug
int xstrTableInit(unsigned long *xstrLUTableAddr,
							  unsigned long xtring_addr,
                       int flags)
{
   unsigned long xstrTableAddr;
   unsigned long xstrTableLen;
	unsigned long nextaddr,paddr,luEntryAddr;
	unsigned i, count;
   xtableEntry xentry;

  	// get table size and count the number of strings
   xmem2root(&xstrTableAddr,xtring_addr,4);
   xstrTableLen = xtring_addr - xstrTableAddr;

   // count strings
	paddr = xstrTableAddr;
   count = 0;
   while(paddr < xtring_addr)
	{
		// look for next occurance of NULL terminator
		paddr = 1 + xmemchr(paddr, 0,
                 (unsigned)(xtring_addr-paddr));
      count++;
   }

	// allocate extended RAM for lookup table
   *xstrLUTableAddr = luEntryAddr =
           xalloc(count*sizeof(xtableEntry));

   // scan through the table for NULL terminators and
   //  make entries in the RAM LU table
  	xentry.padd = xstrTableAddr;
   luEntryAddr = *xstrLUTableAddr;

   for (i = 0; i < count; i++)
	{
		xentry.checksum = 0;
		xentry.checksum = xentry.checksum + (char)(xentry.padd);
		xentry.checksum = xentry.checksum + (char)(xentry.padd>>8);
		xentry.checksum = xentry.checksum + (char)(xentry.padd>>16);

		nextaddr = 1 + xmemchr(xentry.padd, 0,
                 (unsigned)(xtring_addr-xentry.padd));

		xentry.len = (unsigned)(nextaddr-xentry.padd);

		xentry.checksum = xentry.checksum + (char)(xentry.len);
		xentry.checksum = xentry.checksum + (char)(xentry.len>>8);

      root2xmem(luEntryAddr,&xentry,sizeof(xtableEntry));

		luEntryAddr = luEntryAddr + sizeof(xtableEntry);
      xentry.padd = nextaddr;
   }
}

nodebug
int xstrLookup(char* data, unsigned long *xstrLUTabAddr,
               unsigned index, int maxsize)
{
   int i;
   char cs;
   xtableEntry entry;

	// copy lookup table entry to entry.
   xmem2root(&entry, *xstrLUTabAddr + sizeof(xtableEntry)*index,
               sizeof(xtableEntry));

   if (entry.len>maxsize) return -2;

	// validate checksum
	cs = 0;
   for(i=0; i<sizeof(xtableEntry)-1; i++)
   {
		cs = cs + *(char*)((unsigned)&entry+i);
   }
   if (cs != entry.checksum) return -1;

	// copy string to data
   xmem2root(data,entry.padd,entry.len);

   return 0;
}

nodebug
long xstrGet(unsigned long *xstrLUTabAddr,
               unsigned index, int maxsize)
{
   int i;
   char cs;
   xtableEntry entry;

	// copy lookup table entry to entry.
   xmem2root(&entry, *xstrLUTabAddr + sizeof(xtableEntry)*index,
               sizeof(xtableEntry));

   if (entry.len>maxsize) return -2L;

	// validate checksum
	cs = 0;
   for(i=0; i<sizeof(xtableEntry)-1; i++)
   {
		cs = cs + *(char*)((unsigned)&entry+i);
   }
   if (cs != entry.checksum) return -1L;

   return entry.padd;
}

nodebug int timeit(){
// see how a long a look-up takes
   int i;
	unsigned long int	timer, emptyloop;
   unsigned long xstrLUTableAddr;
	char buffer[255];

	xstrTableInit(&xstrLUTableAddr, mystrings,0);

	// compute time for an empty loop
	timer = MS_TIMER;   // get current time in milliseconds
	for(i=0; i<1000u; i++) {
		// empty loop
	}
	emptyloop = MS_TIMER-timer;  // save this value to adjust later measurements

	timer = MS_TIMER;   // get current time in milliseconds
   for(i= 0; i<1000; i++)
   {
	   xstrLookup(buffer,&xstrLUTableAddr,1,sizeof(buffer));
	}
	// # of milliseconds for 1000 iterations = # of microseconds for 1
	printf("\ntime = %8ld microseconds\n", MS_TIMER-timer-emptyloop);
}