/*****************************************************************
 cxbuf.c   Z-World, Inc, 2004

 Simple demonstration of new sample library CBUF.LIB, which includes
 circular buffer API that uses xmem with efficient 16 bit pointers

 Requires new CBUF.LIB with cxbuf functions
*****************************************************************/

/*** Beginheader */
#ifndef _CBUF_LIB_
#use "CBUF.LIB"
#endif
/*** endheader */

#use "XMALLOC.LIB"

unsigned long cxbufPtrPA;

main(){

   static char cxbufSeg, * cxbufPtr;
   int i;

	// intialize xmem buffer
	// circular buffer API needs a 2^n - 1 size
   // buffer plus 9 overhead bytes
	xheapInit(0x1008,2);

	cxbufPtrPA = xmalloc4kalign(0x1008);

	// get the 16 bit address of the buffer
   cxbufPtr  = XPTR_EOFFSET(cxbufPtrPA);
	// get the segment of the buffer
   cxbufSeg =  XPTR_XPC(cxbufPtrPA);

   // initialize circular buffer for 4095 bytes
   cxbuf_init(cxbufPtr,0x0fff,cxbufSeg);

	// put the alphabet in the buffer
   for(i=0;i<26;i++)
   {
		cxbuf_putch( cxbufPtr,65+i,cxbufSeg);
   }

   // shows stats
   printf("free = %d\n", cxbuf_free(cxbufPtr,cxbufSeg));
   printf("used = %d\n\n", cxbuf_used(cxbufPtr,cxbufSeg));

	// print buffer contents
   for(i=0;i<26;i++)
   {
  		printf("%c",cxbuf_getch(cxbufPtr,cxbufSeg));
   }

   // shows stats
   printf("\n\nfree = %d\n", cxbuf_free(cxbufPtr,cxbufSeg));
   printf("used = %d\n", cxbuf_used(cxbufPtr,cxbufSeg));

	// stuff buffer with chars until no more fit
   i = 0;
   while( cxbuf_free(cxbufPtr,cxbufSeg))
   {
		cxbuf_putch( cxbufPtr,32+(i%90),cxbufSeg);
		i++;
   }

   // i is char count in buffer
	printf("\n\n%d characters put\n\n",i);

	// spit out buffer contents until empty
   while(i)
   {
   	i--;
  		printf("%c",cxbuf_getch(cxbufPtr,cxbufSeg));
   }
}