/*****************************************************************
 malloc.c

 Z-World, Inc, 2004

 Simple demonstration of malloc() and free() sample library.
 Read the comment block at the top of malloc.lib
 carefully before using malloc in a real application.

*****************************************************************/

// Array or xmem size to be used as a heap
#define HEAP_SIZE 0x1000    // 4k bytes

// uncomment only one typr of heap
#define XMEM_HEAP
//#define ROOT_HEAP

#use "MALLOC.LIB"
#use "XMALLOC.LIB"

main()
{
	char * p;
   unsigned long paddr;

	// Allocate heap space from xmem
	xheapInit(HEAP_SIZE,2);
   paddr = xmalloc4kalign(HEAP_SIZE);

	// intitialize heap
   heapInit(paddr);

   outputFreeList();  // display freelist

   // malloc two blocks (not checking return codes, but should work,
   //   will output address of 0 if failed)
	printf("malloc'ing 0x100 bytes from %4x\n", (unsigned)malloc(0x100));
	printf("malloc'ing 0x800 bytes from %4x\n", (unsigned)malloc(0x800));

	// malloc another and save address for freeing
	printf("malloc'ing 0x600 bytes from %4x\n",(unsigned)( p = malloc(0x600) ));

   outputFreeList();

	// This malloc should fail
	if(!malloc(0x600))
	{
		printf("malloc failed as expected, insufficient heap space\n");
	}

	// Free last successly malloc'ed 0x600 bytes
	free(p);

   outputFreeList();

	// The same malloc should succeed after block freed
	if(p=malloc(0x10))
	{
		printf("malloc'ed 0x600 bytes from %4x\n",(unsigned)p);
	}
	else
	{
		printf("malloc falled, out of heap space\n");
	}
}

// display the list of free blocks
// there should only be one for this program
root outputFreeList()
 {
	struct free__list *p;

   ENTER_SET_XPC; 	// needed in case heap in xmem

	printf("\n");

   p = alloc__h;
	while(p)
   {
		printf("P = %4x, Size = %04x, next = %04x\n",p,p->Size,p->next);
      p = p->next;
	}
   RESTORE_XPC_EXIT; // needed in case heap in xmem

	printf("\n");
}

