/******************************************************************
 STACKCHECK.C

 Z-World, Inc, 2004

 Demonstrates using an assert macro to check
 if the stack pointer register is getting dangerously close
 to 0xD000, which is the bottom of the stack segment in
 a normal Dynamic C program that doesn't use uC/OS-II.

 Not for use with uC/OS-II, because a uC/OS-II task stack
 might be full before the stack pointer gets close to 0xD000.
 (uC/OS-II provides some stack checking functions.)

******************************************************************/

#define STACKMIN 0xd080  // if stack gets near D000 it is low
#ifndef NDEBUG           // NDEBUG is not defined internally!
#ifndef assert           // assert macro provided in later DC versions.
#define assert(x) \
      ((x)? 0:    \
      printf(#x"  file %s, line %d",__FILE__,__LINE__))
#else
#define assert(ignore)
#endif
#endif

#asm
Stack_Low::
	xor a        // clear carry flag
   ld hl, STACKMIN
   sbc hl,sp    // see if less than min
   jr  c, .stackOK
   ld  hl,0    // return 0 if less
.stackOK:
   ret
#endasm

main()
{
	foo();
}

foo(){
	// remove static and it fails
   // (or replace with auto if default
   // storage class is static as with DC
   // prior to version 8.01
   static char str[0x1000];

   assert(Stack_Low());
	// we haven't written anything to screw up
   // the stack so it will return OK, but
   // the call itself is corrupting data, so this
   // should be treated as a fatal conditon
}






